# Developing Helm Chart

This document provides guidance for developing the Broker Helm Chart.

## Requirements

The following is expected to be in place

* You have Helm >= 3.6 installed
* You have the Kubernetes client tools installed (minimally `kubectl`)
* You have access to a Kubernetes cluster >= 1.16, or Minikube >= 1.28.0
* If using `minikube`, your local machine must have at least 2.5GB of RAM allocated to Docker

## Local usage

Developers can deploy the Helm Chart included in this repository locally, or to an existing Kubernetes cluster which is
running a version that is supported by the Broker Helm Chart. Below, we've included steps for using `minikube`.

### minikube

If you do not have `minikube` installed, it is the recommended path of least resistence to gaining access to a
Kubernetes-like cluster.

While there are a number of ways to install `minikube`, we recommend that you follow the instructions for your operating
system and architecture [here][1].

**macOS**

To install the latest minikube **stable** release on **x86-64 macOS** using **binary download**:

```console
curl -LO https://storage.googleapis.com/minikube/releases/latest/minikube-darwin-amd64
sudo install minikube-darwin-amd64 /usr/local/bin/minikube
```

If you choose to install `minikube` via Homebrew, and you are receiving the following error...

```console
fatal error: runtime: bsdthread_register error
```

...consider installing the binary directly.

**Linux**

To install the latest minikube **stable** release on **x86-64 Linux** using **binary download**:

```console
curl -LO https://storage.googleapis.com/minikube/releases/latest/minikube-linux-amd64
sudo install minikube-linux-amd64 /usr/local/bin/minikube
```

Once `minikube` is installed, you can start it with the following command

```console
minikube start --kubernetes-version=v1.25.3
```

This process may take some time to complete. The tool will inform you of when it is finished starting `minikube`. Also,
your local Kubernetes configuration will be changed to point to this `minikube`. You can ensure that you are pointing
to `minikube` by running the following command.

```console
kubectl get ns
```

On a vanilla `minikube` installation, you should expect to see a smaller number of `Namespace`s than you might see
on a shared tenancy Kubernetes cluster. For example,

```console
$ kubectl get ns
NAME                STATUS   AGE
default             Active   24h
kube-node-lease     Active   24h
kube-public         Active   24h
kube-system         Active   24h
```

If you see many more namespaces, you should be cautious in proceeding because you may be pointed to a non-minikube
instance.

## Create an ImagePullSecret

The image used in the Helm Chart differs depending on whether you are operating like a customer or a VMWare developer.

* Distribution Harbor for customers
* Corporate Harbor for internal developers

The `values-minikube.yaml` defaults to using the Corporate Harbor registry. This registry requires that authentication
be performed to pull the container image. To configure this authentication on your local `minikube` instance, do the
following.

Use `cicd` to create a Secret within that namespace that contains login credentials to Corporate Harbor.

```console
cicd Create-DockerSecret
```

The `regcred` name here is important because it is what is specified in the `vaules-minikube.yaml` file. If you change
this name, you must also change the `global.imagePullSecrets` value in `values-minikube.yaml`.

## Creating an agent

The service deployed via this chart requires an access-key be generated which allows it to communicate with the
Lemans service. To create this agent, use the following command.

```console
cicd Create-Agent
```

## Installing the Chart

The creds-broker can be installed into a cluster using the following commands.

### Vault service

We assume that you need a Vault instance. The following commands will provide you with one.

```console
cicd Install-HelmVault
```

This will install a local copy of vault for you to work with.

When Vault is ready, you will need to create an AppRole that the creds-broker will connect to Vault with.

### Vault AppRole for Broker

The Broker uses an AppRole to connect to Vault. Use the following command to create this role.

```console
cicd Create-VaultAppRole
```

### Broker service

You can then deploy the Broker Helm Chart, preconfigured for a `minikube` environment, with the following command.

```console
cicd Install-HelmBroker
```

The above command should complete rather quickly and print some notes to your terminal about the installation. In the
background, the Pods that compose the chart will be starting up and going live.

## Verifying deployment

You can ensure that the Pods are live by using the `kubectl` command to view the status of their deployment

```console
kubectl get pod -n aria-creds-broker
```

The Chart is successfully deployed when the Pods listed are in the `Ready` state. If Pods do not become ready, use the
following commands to investigate why they are failing to start

```console
kubectl describe pod <POD-NAME>
```

or

```console
kubectl logs -f <POD-NAME>
```

The above commands will usually provide some indication as to what went wrong.

## Adding sample data to show synchronization

This repo includes `cicd` commands to add sample data to Vault to show that the creds-broker correctly synchronizes
information to the Aria SaaS backend (via Lemans)

You can deploy this sample AWS role with the following command

```console
cicd Create-SampleAwsSecret
```

To remove the sample data, use the following command

```console
cicd Delete-SampleAwsSecret
```

## Uninstalling the services

The following steps can be used to uninstall the moving pieces of this service. The following will be addressed

* Removal of Broker
* Removal of Vault
* Cleanup of the Persistent Volumes used by Broker and Vault

### Removing the Broker

The Broker chart can be removed with the following command

```console
cicd Uninstall-HelmBroker
```

This will delete most of the components of the Chart from the namespace. It will not delete the Persistent Volumes used
by the broker though.

If the PVC is not removed, then future re-installations of Broker will re-use this volume instead of creating a new
one from scratch.

To remove these volumes, use the following.

```console
cicd Clean-HelmBroker
```

### Removing Vault

The Vault chart can be removed with the following command

```console
cicd Uninstall-HelmVault
```

This will delete most of the components of the Chart from the namespace. It will not delete the Persistent Volumes used
by the broker though.

If the PVC is not removed, then future re-installations of Vault will re-use this volume instead of creating a new
one from scratch.

To remove these volumes, use the following.

```console
cicd Clean-HelmVault
```

## Vault auto-unseal

The demonstration Vault instance is configured (via the Helm Chart `values-minikube.yaml` file) to auto-initialize and
unseal the Vault instance when the Pod starts; this includes when the Pod is terminated or re-scheduled.

This practice is known to be insecure, but is deliberately done for the purposes of demonstration, development, and
testing. For production, or shared tenancy workloads, it is advised that you come to Credential Broker with an existing
Vault offering.

Do not use the included Vault configuration for production, or shared tenancy, workloads.

If you need to access the generated cluster keys, the following command can be used to view them.

```console
cicd Get-ClusterKeys
```

# References

* [minikube docs][2]
* [minikube installation docs][1]

[1]: https://minikube.sigs.k8s.io/docs/start/
[2]: https://minikube.sigs.k8s.io/docs/
