{{/*
Expand the name of the chart.
*/}}
{{- define "aria-creds-broker.name" -}}
{{- default .Chart.Name .Values.nameOverride | trunc 63 | trimSuffix "-" }}
{{- end }}

{{/*
Create a default fully qualified app name.
We truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec).
If release name contains chart name it will be used as a full name.
*/}}
{{- define "aria-creds-broker.fullname" -}}
{{- if .Values.fullnameOverride }}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" }}
{{- else }}
{{- $name := default .Chart.Name .Values.nameOverride }}
{{- if contains $name .Release.Name }}
{{- .Release.Name | trunc 63 | trimSuffix "-" }}
{{- else }}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" }}
{{- end }}
{{- end }}
{{- end }}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "aria-creds-broker.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end }}

{{/*
Common labels
*/}}
{{- define "aria-creds-broker.labels" -}}
helm.sh/chart: {{ include "aria-creds-broker.chart" . }}
{{ include "aria-creds-broker.selectorLabels" . }}
{{- if .Chart.AppVersion }}
app.kubernetes.io/version: {{ .Chart.AppVersion | quote }}
{{- end }}
app.kubernetes.io/managed-by: {{ .Release.Service }}
{{- end }}

{{/*
Selector labels
*/}}
{{- define "aria-creds-broker.selectorLabels" -}}
app.kubernetes.io/name: {{ include "aria-creds-broker.name" . }}
app.kubernetes.io/instance: {{ .Release.Name }}
{{- end }}

{{/*
Create the name of the service account to use
*/}}
{{- define "broker.serviceAccount.name" -}}
{{- if .Values.broker.serviceAccount.create -}}
    {{ default (include "aria-creds-broker.fullname" .) .Values.broker.serviceAccount.name }}
{{- else -}}
    {{ default "default" .Values.broker.serviceAccount.name }}
{{- end -}}
{{- end -}}

{{/*
Sets extra service account annotations
*/}}
{{- define "broker.serviceAccount.annotations" -}}
  {{- if .Values.broker.serviceAccount.annotations }}
  annotations:
    {{- $tp := typeOf .Values.broker.serviceAccount.annotations }}
    {{- if eq $tp "string" }}
      {{- tpl .Values.broker.serviceAccount.annotations . | nindent 4 }}
    {{- else }}
      {{- toYaml .Values.broker.serviceAccount.annotations | nindent 4 }}
    {{- end }}
  {{- end }}
{{- end -}}

{{/*
Compute if the broker is enabled.
*/}}
{{- define "broker.brokerEnabled" -}}
{{- $_ := set . "brokerEnabled" (or
  (eq (.Values.broker.enabled | toString) "true")
  (and (eq (.Values.broker.enabled | toString) "-") (eq (.Values.global.enabled | toString) "true"))) -}}
{{- end -}}

{{/*
Compute if the broker serviceaccount is enabled.
*/}}
{{- define "broker.brokerServiceAccountEnabled" -}}
{{- $_ := set . "brokerServiceAccountEnabled"
  (and
    (eq (.Values.broker.serviceAccount.create | toString) "true" )
    (or
      (eq (.Values.broker.enabled | toString) "true")
      (eq (.Values.global.enabled | toString) "true"))) -}}
{{- end -}}

{{/*
Compute if the broker service is enabled.
*/}}
{{- define "broker.brokerServiceEnabled" -}}
{{- template "broker.brokerEnabled" . -}}
{{- $_ := set . "brokerServiceEnabled" (and .brokerEnabled (eq (.Values.broker.service.enabled | toString) "true")) -}}
{{- end -}}

{{/*
Sets extra broker Service annotations
*/}}
{{- define "broker.service.annotations" -}}
  {{- if .Values.broker.service.annotations }}
    {{- $tp := typeOf .Values.broker.service.annotations }}
    {{- if eq $tp "string" }}
      {{- tpl .Values.broker.service.annotations . | nindent 4 }}
    {{- else }}
      {{- toYaml .Values.broker.service.annotations | nindent 4 }}
    {{- end }}
  {{- end }}
{{- end -}}

{{/*
imagePullSecrets generates pull secrets from either string or map values.
A map value must be indexable by the key 'name'.
*/}}
{{- define "imagePullSecrets" -}}
{{- with .Values.global.imagePullSecrets -}}
imagePullSecrets:
{{- range . -}}
{{- if typeIs "string" . }}
  - name: {{ . }}
{{- else if index . "name" }}
  - name: {{ .name }}
{{- end }}
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
externalTrafficPolicy sets a Service's externalTrafficPolicy if applicable.
*/}}
{{- define "service.externalTrafficPolicy" -}}
{{- $type := "" -}}
{{- if .serviceType -}}
{{- $type = .serviceType -}}
{{- else if .type -}}
{{- $type = .type -}}
{{- end -}}
{{- if and .externalTrafficPolicy (or (eq $type "LoadBalancer") (eq $type "NodePort")) }}
  externalTrafficPolicy: {{ .externalTrafficPolicy }}
{{- else }}
{{- end }}
{{- end -}}

{{/* Scheme for health check and local endpoint */}}
{{- define "broker.scheme" -}}
{{- if .Values.global.tlsDisable -}}
{{ "http" }}
{{- else -}}
{{ "https" }}
{{- end -}}
{{- end -}}

{{/*
Sets extra ingress annotations
*/}}
{{- define "broker.ingress.annotations" -}}
  {{- if .Values.broker.ingress.annotations }}
  annotations:
    {{- $tp := typeOf .Values.broker.ingress.annotations }}
    {{- if eq $tp "string" }}
      {{- tpl .Values.broker.ingress.annotations . | nindent 4 }}
    {{- else }}
      {{- toYaml .Values.broker.ingress.annotations | nindent 4 }}
    {{- end }}
  {{- end }}
{{- end -}}

{{/*
Sets the broker deployment update strategy
*/}}
{{- define "broker.strategy" -}}
  {{- if .Values.broker.strategy }}
  strategy:
  {{- $tp := typeOf .Values.broker.strategy }}
  {{- if eq $tp "string" }}
    {{ tpl .Values.broker.strategy . | nindent 4 | trim }}
  {{- else }}
    {{- toYaml .Values.broker.strategy | nindent 4 }}
  {{- end }}
  {{- end }}
{{- end -}}

{{/*
Sets extra broker pod annotations
*/}}
{{- define "broker.annotations" -}}
  {{- if .Values.broker.annotations }}
      annotations:
        {{- $tp := typeOf .Values.broker.annotations }}
        {{- if eq $tp "string" }}
          {{- tpl .Values.broker.annotations . | nindent 4 }}
        {{- else }}
          {{- toYaml .Values.broker.annotations | nindent 4 }}
        {{- end }}
  {{- end }}
{{- end -}}

{{/*
Sets extra broker PVC annotations
*/}}
{{- define "broker.persistentVolume.annotations" -}}
  {{- if .Values.broker.persistentVolume.annotations }}
      annotations:
        {{- $tp := typeOf .Values.broker.persistentVolume.annotations }}
        {{- if eq $tp "string" }}
          {{- tpl .Values.broker.persistentVolume.annotations . | nindent 8 }}
        {{- else }}
          {{- toYaml .Values.broker.persistentVolume.annotations | nindent 8 }}
        {{- end }}
  {{- end }}
{{- end -}}

{{/*
Sets the broker affinity for pod placement
*/}}
{{- define "broker.affinity" -}}
  {{- if .Values.broker.affinity }}
      affinity:
        {{ $tp := typeOf .Values.broker.affinity }}
        {{- if eq $tp "string" }}
          {{- tpl .Values.broker.affinity . | nindent 8 | trim }}
        {{- else }}
          {{- toYaml .Values.broker.affinity | nindent 8 }}
        {{- end }}
  {{ end }}
{{- end -}}

{{/*
Sets the broker topologySpreadConstraints for pod placement
*/}}
{{- define "broker.topologySpreadConstraints" -}}
  {{- if .Values.broker.topologySpreadConstraints }}
      topologySpreadConstraints:
        {{ $tp := typeOf .Values.broker.topologySpreadConstraints }}
        {{- if eq $tp "string" }}
          {{- tpl .Values.broker.topologySpreadConstraints . | nindent 8 | trim }}
        {{- else }}
          {{- toYaml .Values.broker.topologySpreadConstraints | nindent 8 }}
        {{- end }}
  {{ end }}
{{- end -}}

{{/*
Sets the broker toleration for pod placement
*/}}
{{- define "broker.tolerations" -}}
  {{- if .Values.broker.tolerations }}
      tolerations:
      {{- $tp := typeOf .Values.broker.tolerations }}
      {{- if eq $tp "string" }}
        {{ tpl .Values.broker.tolerations . | nindent 8 | trim }}
      {{- else }}
        {{- toYaml .Values.broker.tolerations | nindent 8 }}
      {{- end }}
  {{- end }}
{{- end -}}

{{/*
Sets the broker node selector for pod placement
*/}}
{{- define "broker.nodeselector" -}}
  {{- if .Values.broker.nodeSelector }}
      nodeSelector:
      {{- $tp := typeOf .Values.broker.nodeSelector }}
      {{- if eq $tp "string" }}
        {{ tpl .Values.broker.nodeSelector . | nindent 8 | trim }}
      {{- else }}
        {{- toYaml .Values.broker.nodeSelector | nindent 8 }}
      {{- end }}
  {{- end }}
{{- end -}}

{{/*
securityContext for the broker pod level.
*/}}
{{- define "broker.securityContext.pod" -}}
  {{- if .Values.broker.securityContext.pod }}
      securityContext:
        {{- $tp := typeOf .Values.broker.securityContext.pod }}
        {{- if eq $tp "string" }}
          {{- tpl .Values.broker.securityContext.pod . | nindent 8 }}
        {{- else }}
          {{- toYaml .Values.broker.securityContext.pod | nindent 8 }}
        {{- end }}
  {{- else if not .Values.global.openshift }}
      securityContext:
        runAsNonRoot: true
        runAsGroup: {{ .Values.broker.gid | default 1000 }}
        runAsUser: {{ .Values.broker.uid | default 100 }}
        fsGroup: {{ .Values.broker.gid | default 1000 }}
  {{- end }}
{{- end -}}

{{/*
Sets the container resources if the user has set any.
*/}}
{{- define "broker.resources" -}}
  {{- if .Values.broker.resources -}}
          resources:
{{ toYaml .Values.broker.resources | indent 12}}
  {{ end }}
{{- end -}}

{{/*
securityContext for the broker container level.
*/}}
{{- define "broker.securityContext.container" -}}
  {{- if .Values.broker.securityContext.container}}
          securityContext:
            {{- $tp := typeOf .Values.broker.securityContext.container }}
            {{- if eq $tp "string" }}
              {{- tpl .Values.broker.securityContext.container . | nindent 12 }}
            {{- else }}
              {{- toYaml .Values.broker.securityContext.container | nindent 12 }}
            {{- end }}
  {{- else if not .Values.global.openshift }}
          securityContext:
            allowPrivilegeEscalation: false
            capabilities:
              drop:
                - ALL
  {{- end }}
{{- end -}}

{{/*
Inject extra environment vars in the format key:value, if populated
*/}}
{{- define "broker.extraEnvironmentVars" -}}
{{- if .extraEnvironmentVars -}}
{{- range $key, $value := .extraEnvironmentVars }}
- name: {{ printf "%s" $key | replace "." "_" | upper | quote }}
  value: {{ $value | quote }}
{{- end }}
{{- end -}}
{{- end -}}

{{/*
Inject extra environment populated by secrets, if populated
*/}}
{{- define "broker.extraSecretEnvironmentVars" -}}
{{- if .extraSecretEnvironmentVars -}}
{{- range .extraSecretEnvironmentVars }}
- name: {{ .envName }}
  valueFrom:
   secretKeyRef:
     name: {{ .secretName }}
     key: {{ .secretKey }}
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
Set's which additional volumes should be mounted to the container.
*/}}
{{- define "broker.mounts" -}}
  {{- range .Values.broker.extraVolumes }}
            - name: userconfig-{{ .name }}
              readOnly: true
              mountPath: {{ .path | default "/broker/userconfig" }}/{{ .name }}
  {{- end }}
  {{- if .Values.broker.volumeMounts }}
    {{- toYaml .Values.broker.volumeMounts | nindent 12}}
  {{- end }}
{{- end -}}

{{/*
Iterates over any extra volumes the user may have specified (such as a secret
with TLS).
*/}}
{{- define "broker.volumes" -}}
  {{- range .Values.broker.extraVolumes }}
        - name: userconfig-{{ .name }}
          {{ .type }}:
          {{- if (eq .type "configMap") }}
            name: {{ .name }}
          {{- else if (eq .type "secret") }}
            secretName: {{ .name }}
          {{- end }}
            defaultMode: {{ .defaultMode | default 420 }}
  {{- end }}
  {{- if .Values.broker.volumes }}
    {{- toYaml .Values.broker.volumes | nindent 8}}
  {{- end }}
{{- end -}}

{{/*
Create list of Java opts that are specific to configuration of the service.
*/}}
{{- define "broker.extJavaOpts" -}}
{{- $opts := list (print "-DbrokerId=" .Values.broker.id) }}
{{- $opts = append $opts (print "-DappRoleId=" (.Values.broker.vault.appRoleId | default .Values.broker.id)) }}
{{- $opts = append $opts (print "-DlemansAccessKey=" .Values.broker.lemans.accessKey) }}
{{- $opts = append $opts (print "-DlemansDataVolume=" .Values.broker.lemans.dataVolume) }}
{{- $opts = append $opts (print "-DlemansServiceUri=" .Values.broker.lemans.serviceUrl) }}
{{- $opts = append $opts (print "-Dserver.port=" .Values.broker.port) }}
{{- $opts = append $opts (print "-Dvault.host=" .Values.broker.vault.host) }}
{{- $opts = append $opts (print "-Dvault.port=" .Values.broker.vault.port) }}
{{- $opts = append $opts (print "-Dvault.readTimeout=" .Values.broker.vault.readTimeout) }}
{{- $opts = append $opts (print "-Dvault.scheme=" .Values.broker.vault.scheme) }}
{{- $opts = concat $opts (splitList " " .Values.broker.extJavaOpts) }}
{{- quote (join " " $opts) -}}
{{- end -}}